import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/***************   C l a s s :    G a m e p a d   *******************************
 * Gampad implements a screen window, which shows and manages a game score		*
 * counter and a stopwatch and implementes functions for key and mouse entries.	*
 * It also provides some convenience functions for game programming.			*
 * 																				*
 * Gamepad requires the class StopWatch to be located in the same directory.	*
 ********************************************************************************/
 
import de.hamster.debugger.model.Territorium;import de.hamster.debugger.model.Territory;import de.hamster.model.HamsterException;import de.hamster.model.HamsterInitialisierungsException;import de.hamster.model.HamsterNichtInitialisiertException;import de.hamster.model.KachelLeerException;import de.hamster.model.MauerDaException;import de.hamster.model.MaulLeerException;import de.hamster.model.MouthEmptyException;import de.hamster.model.WallInFrontException;import de.hamster.model.TileEmptyException;import de.hamster.debugger.model.Hamster;public class Gamepad extends JFrame
        implements KeyListener, ActionListener
{
	/* ------- Functions for keyboard handling -----------------------------------*/
	
    // Return remembered last typed key character and reset this to 0.
    // So modifiers like 'shift' and 'alt' are taken into account. This
    // Only the last typed key is remembered, every subsequent key overrides
    // any previous key.
    // Be aware that a character and a key code (see following method)
    // is something different. A character can be e.g. 'a' but also 'A',
    // where as a key code is a system internal number of a keyboard key.
    public char grabLastTypedKeyChar () {
    	char c=lastTypedKeyChar;
    	lastTypedKeyChar=0;
    	return c;
    }
    
    // Allows questioning, whether a key with a specific code is currently pressed.
    // This gives the possibility to check for continously pressed keys, as one
    // would use it in a jump-and-run game, instead of testing for individual
    // key strokes. Use this also, if you want to check for multiple keyes pressed
    // simultaneously.
    // If you want to learn about key codes and corresponding keys, simply
    // create a Gamepad with the showDetail parameter set to 'true', and then
    // observe the log output when you press keys.
    // B.t.w: here we only handle key codes in the range from 0 to 255. All other
    // values return a false right away.
    public boolean keyIsPressed(int keyCode) {
    	if (keyCode<0 || keyCode>255) return false;
    	return keyPressedTable[keyCode];
    }

	/* ------- Convenience methods  --------------------------------------*/
    
    // Although pausing is not an essential function of a game pad,
    // it is an essential function of a game for slowing down the flow.
    // Thus we add this here, to make pausing a convenient thing.
    public void pause(int mseconds) {
    	if (mseconds>0) {
	    	try {
			    Thread.sleep(mseconds);
			} catch(InterruptedException ex) {
	    		Thread.currentThread().interrupt();
			}
		}
	}

	/* ------- Methods for game score -----------------------------------*/
    // The Gamepad provides a game score counter including its continuous display.
    // It can be controlled via this interface:
    
    // getScore() returns the currently remembered score value
    public long getScore() {
		return score;
	}

    // setScore() sets the score to the given value
    public void setScore(long l) {
		score=l;
		scoreDisp.setText(String.valueOf(score));
	}

    // addScore() adds the given points to the remembered score. I guess this
    // is the most frequently used call for programming a game.
    // A negative value is allowed too.
    public void addScore(long i) {
		score+=i;
		scoreDisp.setText(String.valueOf(score));
	}

	/* ------- Methods for timer/stopwatch -----------------------------------*/
    // The Gamepad provides a stopwatch including its continuous display.
    // It can be controlled via this interface:

	// startStopWatch() starts the stopwatch where ever it was stopped before.
	// If it is currently running, startStopWatch() resets it to 0, and starts it.
    public void startStopWatch() {
		watch.start();
	}

	// stopStopWatch() stops the stopwatch, as you would do on a real stopwatch.
	// If it is already stopped, nothing happens.
    public void stopStopWatch() {
		watch.stop();
		timeDisp.setText(watch.getTimeMS());
	}

	// resetStopWatch() resets the stopwatch to 0, without changing the running
	// state, i.e. if it was running, it continues running from 0.
    public void resetStopWatch() {
		watch.reset();
		timeDisp.setText(watch.getTimeMS());
	}

	/* ------------- log method ------------------------------------------------*/
	// Use this, to print out something on the extended Gamepad display.
	// The log string is always termiantd by a line feed.
	public void log(String s) {
		displayArea.append(s + newline);
    	displayArea.setCaretPosition(displayArea.getDocument().getLength());
    }

	
	/* --------------------------------------------------------------------------*/
	/* -----Class internal stuff: -----------------------------------------------*/    
    
	private StopWatch watch = new StopWatch();

	private static final String newline = System.getProperty("line.separator");

	// 'Swing' text area for displaying additinal messages, e.g. key stroke codes
    private JTextArea displayArea;

	// 'Swing' labels and panels for data display
 	private	JLabel scoreLabel, scoreDisp, timeLabel, timeDisp;
 	private JPanel timeBox, scoreBox, scoreAndTimeBox;
          
	// Remember the last typed key character
	private char lastTypedKeyChar=0;

	// Remember the 'pressed' status of each key code
    private boolean[] keyPressedTable= new boolean[256];
    
    private long score=0;
    
    public Gamepad(String name) {
    	this(name, true);
    }
    
    public Gamepad(String name, boolean showDetails) {
        super(name);
        
        // main activities in the following section: assemble 
        // the window (JFrame)
        
    	// allow the JFrame to stop the whole program: 
    	setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
         
        // define the character fonts for the JLabels:
        Font labelFont= new Font("SansSerif", Font.BOLD, 20);
        Font dispFont= new Font("SansSerif", Font.BOLD, 40);

    	// Create two JLables for the time display:
		timeLabel = new JLabel("Time", JLabel.LEFT);
 		timeLabel.setFont(labelFont);
		timeDisp = new JLabel(watch.getTimeMS(), JLabel.LEFT);
 		timeDisp.setFont(dispFont);
 		timeDisp.setToolTipText("Game Timer");
		
    	// timeBox = timeLabel + timeDisp, vertically alligned
		timeBox = new JPanel();
		timeBox.setLayout(new BorderLayout(0,0));
		timeBox.add(timeLabel, BorderLayout.PAGE_START);
		timeBox.add(timeDisp, BorderLayout.PAGE_END);

    	// Create two JLables for the score display:
 		scoreLabel = new JLabel("Score", JLabel.RIGHT);
 		scoreLabel.setFont(labelFont);
		scoreDisp = new JLabel(String.valueOf(score), JLabel.RIGHT);
 		scoreDisp.setFont(dispFont);
 		scoreDisp.setToolTipText("Game Score");
		
    	// scoreBox = scoreLabel + scoreDisp, vertically alligned
		scoreBox = new JPanel();
		scoreBox.setLayout(new BorderLayout(0,0));
		scoreBox.add(scoreLabel, BorderLayout.PAGE_START);
		scoreBox.add(scoreDisp, BorderLayout.PAGE_END);
		
    	// scoreAndTimeBox = timeBox + scoreBox, horizontally alligned
		scoreAndTimeBox = new JPanel();
		scoreAndTimeBox.setLayout(new BorderLayout(0,0));
		scoreAndTimeBox.add(timeBox, BorderLayout.WEST);
		scoreAndTimeBox.add(scoreBox, BorderLayout.EAST);
		
		// Let's take this box to handle the key event callbacks:         
        scoreAndTimeBox.addKeyListener(this);
        scoreAndTimeBox.requestFocusInWindow();
         
		// for the detail view we also need a text area:
        displayArea = new JTextArea();
        displayArea.setEditable(false);
        JScrollPane scrollPane = new JScrollPane(displayArea);
        scrollPane.setPreferredSize(new Dimension(375, 225));

		// Use standard layout: BorderLayout
		// No we glue all this together to the Gamepad JFrame:
		setLayout(new BorderLayout(20,20));
		add(scoreAndTimeBox, BorderLayout.PAGE_START);
		add(scrollPane, BorderLayout.CENTER);

		// Display scrollPane only if showDetails is set:
		scrollPane.setVisible(showDetails);

   		pack();
   		if (showDetails) setSize(400,400);
   		else setSize(400,100);
   		
		// Let's bring it to light, finally:
		setVisible(true);
		
		// The scoreAndTimeBox gets the focus, so that it receives the key events.
        scoreAndTimeBox.requestFocusInWindow();

		// Start a timer to trigger the actionPerformed callback every 333 msec.
		// so that the stopwatch is updated regularily:
		new Timer(333, this).start();

		log ("INFO: Gamepad V1.0 (2013-11-30) initialized.");
	}
     
         
	/* ----- Event handlers for keyboard events -----------------------------------------*/    

    // Handle the key typed event from the text field:
    public void keyTyped(KeyEvent e) {
        lastTypedKeyChar=e.getKeyChar();
		String s="INFO: grabLastTypedKeyChar()=='" + lastTypedKeyChar + "'";
        displayArea.append(s + newline);
        displayArea.setCaretPosition(displayArea.getDocument().getLength());
    }
     
    // Handle the key pressed event from the text field:
    public void keyPressed(KeyEvent e) {
        int i=e.getKeyCode();
        if (i>0 && i<256)	{
        	keyPressedTable[i]=true;
   			String s="INFO: keyIsPressed(" + String.valueOf(i) + ")==true";
	        displayArea.append(s + newline);
    	    displayArea.setCaretPosition(displayArea.getDocument().getLength());
		}
    }
     
    // Handle the key released event from the text field:
    public void keyReleased(KeyEvent e) {
        int i=e.getKeyCode();
        if (i>0 && i<256) {
        	keyPressedTable[i]=false;
   			String s="INFO: keyIsPressed(" + String.valueOf(i) + ")==false";
	        displayArea.append(s + newline);
    	    displayArea.setCaretPosition(displayArea.getDocument().getLength());
		}
    }
     
	/* ----- Event handler(s) for mouse or Swing timer events: ---------------------------*/    

    // Handle the timer event (or button click):
    public void actionPerformed(ActionEvent e) {
		timeDisp.setText(watch.getTimeMS());
    }

} // Gamepad

/* Todos:
 - Limit size of JTextArea test (memory leak)
 - Analyze security manager conflicts when accessing files, at least on MacOS
 - Add sound features
 - Differentiate three or four log levels 
 - Optimize border and layout
 - 
 */
